# -*- coding: utf-8 -*-
# raumplanerModell-ohne-Auswahl.py
## Variante zum Erarbeiten der Auswahl

"""Eine eigene Modellklasse für die Moebel-Klassen
übernimmt die gesamte Verwaltung der erzeugten Möbel
einschließlich Speichern und Laden,
sowie Auswahl
Die Klasse ist als Singleton modelliert"""

import pickle # Persistenz realisieren

from badewanne import Badewanne
from bett import Bett
from dusche import Dusche
from herd import Herd
from klosett import Klosett
from sessel import Sessel
from schrank import Schrank
from sofa import Sofa
from stuhl import Stuhl
from tisch import Tisch
from ovalerTisch import OvalerTisch

### ---------------------------------------------------
class RaumplanerModell(object):
    '''Ist als Singleton modelliert'''

    __raumplanerModell = None

    @staticmethod
    def GibRaumplanerModell():
        if RaumplanerModell.__raumplanerModell == None:
            RaumplanerModell.__raumplanerModell=RaumplanerModell()
        return RaumplanerModell.__raumplanerModell
    
    def __init__(self):
        """Konstruktor"""
        ## Fuer Singleton: ----------------------------------------
        if RaumplanerModell.__raumplanerModell != None:
            raise Exception('Konstruktor nicht direkt aufrufen')
        ## --------------------------------------------------------
        self.__alleMoebel = []
        

    #### ----- Erzeugen -----
    def Neu(self, klasse, *par, **kwargs):
        if klasse.__class__==str:
            klasse=eval(klasse)
        neu=klasse(*par, **kwargs)
        self.__alleMoebel.append(neu)
        return neu

    # Loeschen s.u.

    #### ----- sichern und laden -----
    def Speichere(self, dateiname):
        """Objektdaten werden in Datei geschrieben"""
        try:
            with open(dateiname, 'wb') as objekt_datei:
                pickle.dump(self.__alleMoebel, objekt_datei)
                return 'OK'
        except IOError as ioerr:
            return 'Dateifehler: ' + str(ioerr)
        except pickle.PicklingError as perr:
            return 'Pickle-Fehler: ' + str(perr)

    def Lade(self, dateiname):
        """Objektdaten werden aus der Datei gelesen"""
        
        ## ToDo: vorher alle vorhandenen Moebelobjekte löschen

        try:
            with open(dateiname, 'rb') as objekt_datei:
                self.__alleMoebel = pickle.load(objekt_datei)
                for moebel in self.__alleMoebel:
                    if moebel.GibSichtbar():
                        moebel.Zeige()
                return 'OK'
        except IOError:
            return 'Datei nicht auffindbar!'
        except AttributeError as ae:
            return 'AttributeError: ' + str(ae)
        except pickle.UnpicklingError as perr:
            return 'Pickle-Fehler: ' + str(perr)

    def BildExport(self, dateiname):
        """Bild wird in Bild-Datei geschrieben"""
        erg=Zeichenflaeche.GibZeichenflaeche().BildExport(dateiname)
        print(erg)

    #### ----- Auswahl -----
    def Waehle(self, index=None):
        '''wählt das Moebel indexbasiert aus'''
        ## ToDo
        pass

    def HatAusgewaehltes(self):
        '''gibt zurück, ob ein Moebel ausgewählt ist'''
        ## ToDo
        pass        

    def GibAusgewaehltes(self):
        '''gibt das aktuell ausgewaehlte Moebel zurück'''
        ## ToDo
        pass

    def LoescheAusgewaehltes(self):
        '''löscht das aktuell ausgewaehlte Moebel'''
        ## ToDo
        pass

    #### ----- Aktionen -----
    def BewegeHorizontal(self, weite):
        """Bewegt das Ausgewaehlte horizontal"""
        if self.HatAusgewaehltes():
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.BewegeHorizontal(weite)
        
    def BewegeVertikal(self, weite):
        """Bewegt das Ausgewaehlte vertikal"""
        if self.HatAusgewaehltes():
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.BewegeVertikal(weite)
        
    def AendereBreite(self, neueBreite):
        """Ändert die Farbe des Ausgewaehlten"""
        if self.HatAusgewaehltes():
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.AendereBreite(neueBreite)
    
    def AendereTiefe(self, neueTiefe):
        """Ändert die Farbe des Ausgewaehlten"""
        if self.HatAusgewaehltes():
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.AendereTiefe(neueTiefe)
    
    def Drehe(self, winkel):
        """Dreht das Ausgewaehlte"""
        if self.HatAusgewaehltes():
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.Drehe(winkel)
        
    def AendereFarbe(self, neueFarbe):
        """Ändert die Farbe des Ausgewaehlten"""
        if self.HatAusgewaehltes():
            gewaehlt=self.GibAusgewaehltes()
            gewaehlt.AendereFarbe(neueFarbe)

    def GibAlleMoebel(self):
        return self.__alleMoebel

### ---------------------------------------------------
import wx
from grafikfenster import *
class MoebelApp(wx.App):
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.modell=RaumplanerModell.GibRaumplanerModell()
        ##self.TestAnwendung()
        return True

    def TestAnwendung(self):
        global modell, schrank, sessel, stuhl, tisch
        modell=self.modell
        schrank=self.modell.Neu(Schrank,280, 40, 100, 60, 90, 'black', True)
        stuhl=self.modell.Neu(Stuhl,10, 30, 40, 40, 270, 'blue', True)
        tisch=self.modell.Neu(Tisch,60, 20, 120, 60, 0, 'red', True)
        sessel=self.modell.Neu(Sessel, 100, 130, 60, 60, 0, 'orange', True)
        sofa=self.modell.Neu(Sofa, 70, 290, 120, 60, 180, 'orange', True)
        ovaler=self.modell.Neu(OvalerTisch, 70, 200, 120, 80, 0, 'brown', True)

### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
